/*
#######################################

Copyright 2019 Micro-Photon-Devices s.r.l.

SOFTWARE PRODUCT: PDM-IR_SDK

Micro-Photon-Devices (MPD) expressly disclaims any warranty for the SOFTWARE PRODUCT.
The SOFTWARE PRODUCT is provided 'As Is' without any express or implied warranty of any kind,
including but not limited to any warranties of merchantability, noninfringement, or
fitness of a particular purpose. MPD does not warrant or assume responsibility for the
accuracy or completeness of any information, text, graphics, links or other items contained
within the SOFTWARE PRODUCT. MPD further expressly disclaims any warranty or representation
to Authorized Users or to any third party.
In no event shall MPD be liable for any damages (including, without limitation, lost profits,
business interruption, or lost information) rising out of 'Authorized Users' use of or inability
to use the SOFTWARE PRODUCT, even if MPD has been advised of the possibility of such damages.
In no event will MPD be liable for loss of data or for indirect, special, incidental,
consequential (including lost profit), or other damages based in contract, tort
or otherwise. MPD shall have no liability with respect to the content of the
SOFTWARE PRODUCT or any part thereof, including but not limited to errors or omissions contained
therein, libel, infringements of rights of publicity, privacy, trademark rights, business
interruption, personal injury, loss of privacy, moral rights or the disclosure of confidential
information.

#######################################
*/

//#include "stdafx.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "PDM-IR_SDK.h"
#if defined(_WIN32) 
	#include <Windows.h>
#else 
	#include <unistd.h>
#endif



int exit(void)
{
	#if defined(_WIN32) 
		system("PAUSE");
	#else 
		printf("Press <RETURN> to continue...");
		while(getchar() != '\n');
	#endif
	return 0;
}

void manageResult(RESULT res)
{
	switch (res)
	{
	case RESULT_DATA_ERROR:
		printf("error setting parameters.\n");
		return;
	case RESULT_DATA_OUT_LIMIT_HIGH:
		printf("parameter value is higher than max value.\n");
		return;
	case RESULT_DATA_OUT_LIMIT_LOW:
		printf("parameter value is lower than min value.\n");
		return;
	case RESULT_ERROR_COMMUNICATION:
		printf("error communicating with the module.\n");
		return;
	case RESULT_OK:
		return;
	}

}

int main()
{
	UINT16 sn[50];
	UINT16 devfound= 35;
	int i;
	MODULECONFIG config;
	char configName[17];
	MODULEINFO	info;
	UINT32 temp32, temp32_2, temp32_3, temp32_4, temp32_5;
	UINT8 temp8;
	STATUSBITS moduleStatus;
	STATUSBITS moduleTemperature;
	STATUSBITS moduleSpad;
	STATUSBITS moduleGate;
	STATUSWARMUP warningUp;
	UINT32 errorModule;

	printf("*******************************************************\n");
	printf("               PDM-IR Test program\n");
	printf("*******************************************************\n\n\n\n");

	printf("*** SEARCHING DEVICES...\n");
	PDMIR_SearchDevices(sn, &devfound);
	printf("    number of devices found: %d", devfound);
	if (devfound == 0)
		return exit();
	printf(", with the following serial numbers : ");
	for (i = 0; i < devfound-1; i++)
		printf("%d,", sn[i]);
	printf("%d\n", sn[i]);

	printf("\n\n\n");
	printf("*** OPENING COMMUNICATION WITH SERIAL NUMBER %d...\n", sn[0]);
	switch (PDMIR_OpenCommunication(sn[0]))
	{
	case RESULT_OK:
		printf("    module communication correctly opened.\n");
		break;
	case RESULT_ERROR_COMMUNICATION:
		printf("    error communicating with the module.\n");
		return exit();
	default:
		printf("    unexpected output value. Contact MPD.\n");
		return exit();
	}
	
	printf("*** SETUP PARAMETERS\n");
	printf("    Set Amplitude [mV] = ");
	#ifndef _WIN32
		scanf("%hd", &config.Amplitude);
	#else 
		scanf_s("%hd", &config.Amplitude);
	#endif
	manageResult(PDMIR_SetAmplitude(sn[0], config.Amplitude));
	PDMIR_GetAmplitude(sn[0], &config.Amplitude);
	printf("    Read Amplitude = %d mV\n", config.Amplitude);
	
	printf("\n    Set Hold Off [ns] = ");
	#ifndef _WIN32
		scanf("%lu", &config.HoldOff);
	#else 
		scanf_s("%lu", &config.HoldOff);
	#endif
	config.HoldOffLevel = HOLDOFFTYPE_EDGE;
	manageResult(PDMIR_SetHoldOff(sn[0], config.HoldOff, config.HoldOffLevel));
	PDMIR_GetHoldOff(sn[0], &config.HoldOff, &config.HoldOffLevel);
	printf("    Read Hold Off= %lu ns, %s type = %d\n", config.HoldOff, (config.HoldOffLevel == HOLDOFFTYPE_EDGE ? "EDGE" : "LEVEL"), config.HoldOffLevel);

	printf("\n    Temperature Values:\n");
	printf("        0: High temperature\n");
	printf("        1: Medium temperature\n");
	printf("        2: Low temperature\n");
	printf("        3: Lowest temperature\n");
	printf("    Set Temperature = ");
	#ifndef _WIN32
		scanf("%d", (int *)(&config.Temperature));
	#else 
		scanf_s("%d", (int *)(&config.Temperature));
	#endif
	manageResult(PDMIR_SetTemperature(sn[0], config.Temperature));
	PDMIR_GetTemperature(sn[0], &config.Temperature);
	printf("    Read Temperature = %d\n", config.Temperature);

	printf("\n    Gate Mode:\n");
	printf("        0: internal\n");
	printf("        1: external\n");
	printf("    Set Gate Mode = ");
	#ifndef _WIN32
		scanf("%d", (int *)&config.GateMode);
	#else 
		scanf_s("%d", (int *)&config.GateMode);
	#endif
	manageResult(PDMIR_SetGateMode(sn[0], config.GateMode));
	PDMIR_GetGateMode(sn[0], &config.GateMode);
	printf("    Read Gate Mode = %d\n", config.GateMode);

	printf("\n    Gate Shape:\n");
	printf("        0: fixed gate\n");
	printf("        1: free gate\n");
	printf("        2: free running\n");
	printf("    Set Gate Mode = ");
	#ifndef _WIN32
		scanf("%d", (int *)&config.GateShape);
	#else 
		scanf_s("%d", (int *)&config.GateShape);
	#endif
	manageResult(PDMIR_SetGateShape(sn[0], config.GateShape));
	PDMIR_GetGateShape(sn[0], &config.GateShape);
	printf("    Read Gate Shape = %d\n", config.GateShape);

	printf("\n    Trigger Edge:\n");
	printf("        0: low to high\n");
	printf("        1: high to low\n");
	printf("    Set Trigger In Edge = ");
	#ifndef _WIN32
		scanf("%d", (int *)&config.TriggerEdge);
	#else 
		scanf_s("%d", (int *)&config.TriggerEdge);
	#endif
	manageResult(PDMIR_SetTriggerEdge(sn[0], config.TriggerEdge));
	PDMIR_GetTriggerEdge(sn[0], &config.TriggerEdge);
	printf("    Read Trigger Edge = %d\n", config.TriggerEdge);

	printf("\n    Set Trigger In Threshold [mV] = ");
	#ifndef _WIN32
		scanf("%d", (int*)&config.TriggerInTh);
	#else 
		scanf_s("%d", (int*)&config.TriggerInTh);
	#endif
	manageResult(PDMIR_SetTriggerInTh(sn[0], config.TriggerInTh));
	PDMIR_GetTriggerInTh(sn[0], &config.TriggerInTh);
	printf("    Read Trigger In Threshold = %d mV\n", config.TriggerInTh);

	printf("\n    Set Ton [ns] = ");
	#ifndef _WIN32
		scanf("%lu", &config.Ton);
	#else 
		scanf_s("%lu", &config.Ton);
	#endif
	manageResult(PDMIR_SetTon(sn[0], config.Ton));
	PDMIR_GetTon(sn[0], &config.Ton);
	printf("    Read Ton = %d ns\n", config.Ton);

	printf("\n    Set Internat Trigger Frequency [Hz] = ");
	#ifndef _WIN32
		scanf("%lu", &config.Frequency);
	#else 
		scanf_s("%lu", &config.Frequency);
	#endif
	manageResult(PDMIR_SetFrequency(sn[0], config.Frequency));
	PDMIR_GetFrequency(sn[0], &config.Frequency);
	printf("   Read Frequency = %d Hz\n", config.Frequency);

	printf("\n*** OUTPUT PARAMETERS\n");

	printf("\n    Out mode:\n");
	printf("        0: Trigger Gate\n");
	printf("        1: photon out\n");
	printf("        2: hold off\n");
	printf("        3: valid gate\n");
	printf("    Set TTL out mode = ");
	#ifndef _WIN32
		scanf("%d", (int*)&config.TtlOut);
	#else 
		scanf_s("%d", (int*)&config.TtlOut);
	#endif
	manageResult(PDMIR_SetTTLOut(sn[0], config.TtlOut));
	PDMIR_GetTTLOut(sn[0], &config.TtlOut);
	printf("   Read Ttl Out = %d\n", config.TtlOut);

	printf("\n    Out mode:\n");
	printf("        0: Trigger Gate\n");
	printf("        1: photon out\n");
	printf("        2: hold off\n");
	printf("        3: valid gate\n");
	printf("    Set NIM out mode = ");
	#ifndef _WIN32
		scanf("%d", (int*)&config.NimOut);
	#else 
		scanf_s("%d", (int*)&config.NimOut);
	#endif
	manageResult(PDMIR_SetNIMOut(sn[0], config.NimOut));
	PDMIR_GetNIMOut(sn[0], &config.NimOut);
	printf("   Read Nim Out = %d\n", config.NimOut);

	printf("\n*** DELAY PARAMETERS\n");

	printf("    Set Trigger In Delay [ns] = ");
	#ifndef _WIN32
		scanf("%hu", (short*)&config.DelayTriggerIn);
	#else 
		scanf_s("%hu", (short*)&config.DelayTriggerIn);
	#endif
	manageResult(PDMIR_SetDelayTriggerIn(sn[0], config.DelayTriggerIn));
	PDMIR_GetDelayTriggerIn(sn[0], &config.DelayTriggerIn);
	printf("   Read Trigger In Delay = %d ns\n", config.DelayTriggerIn);

	printf("    Set Internal Trigger Delay [ns] = ");
	#ifndef _WIN32
		scanf("%hu", (short*)&config.DelayTriggerInternal);
	#else 
		scanf_s("%hu", (short*)&config.DelayTriggerInternal);
	#endif
	manageResult(PDMIR_SetDelayTriggerInternal(sn[0], config.DelayTriggerInternal));
	PDMIR_GetDelayTriggerInternal(sn[0], &config.DelayTriggerInternal);
	printf("   Read Internal Trigger Delay = %d ns\n", config.DelayTriggerInternal);

	printf("\n*** COUNTER STATUS\n");
	config.CounterIntegrationTime = 1000;
	PDMIR_SetCounterIntegrationTime(sn[0], config.CounterIntegrationTime);
	PDMIR_GetCounterIntegrationTime(sn[0], &config.CounterIntegrationTime);
	printf("Counter Integration Time = %d\n", config.CounterIntegrationTime);

	PDMIR_GetCounterAuxInStatus(sn[0], &config.CounterAuxInStatus);
	printf("CounterAuxInStatus = %d\n", config.CounterAuxInStatus);

	PDMIR_GetCounterAvalancheStatus(sn[0], &config.CounterAvalancheStatus);
	printf("CounterAvalancheStatus = %d\n", config.CounterAvalancheStatus);

	PDMIR_GetCounterInternalTriggerStatus(sn[0], &config.CounterInternalTriggerStatus);
	printf("CounterInternalTriggerStatus = %d\n", config.CounterInternalTriggerStatus);

	PDMIR_GetCounterTriggerInStatus(sn[0], &config.CounterTriggerInStatus);
	printf("CounterTriggerInStatus = %d\n", config.CounterTriggerInStatus);

	PDMIR_GetCounterValidGateStatus(sn[0], &config.CounterValidGateStatus);
	printf("CounterValidGateStatus = %d\n", config.CounterValidGateStatus);
	
	manageResult(PDMIR_GetAllCountersStatus(sn[0], &config.CounterAvalancheStatus, &config.CounterValidGateStatus, &config.CounterAuxInStatus, &config.CounterTriggerInStatus, &config.CounterInternalTriggerStatus));
	printf("CounterAvalancheStatus = %d\n", config.CounterAvalancheStatus);
	printf("CounterValidGateStatus = %d\n", config.CounterValidGateStatus);
	printf("CounterAuxInStatus = %d\n", config.CounterAuxInStatus);
	printf("CounterTriggerInStatus = %d\n", config.CounterTriggerInStatus);
	printf("CounterInternalTriggerStatus = %d\n", config.CounterInternalTriggerStatus);

#if defined(_WIN32) 
	Sleep(config.CounterIntegrationTime * 2);
#else
	usleep(config.CounterIntegrationTime * 2000);
#endif

	PDMIR_GetCounterAuxInLastValue(sn[0], &temp8, &temp32);
	printf("CounterAuxInLastValue = %lu at index %d\n", temp32, temp8);

	PDMIR_GetCounterAvalancheLastValue(sn[0], &temp8, &temp32);
	printf("CounterAvalancheLastValue = %lu at index %d\n", temp32, temp8);

	PDMIR_GetCounterInternalTriggerLastValue(sn[0], &temp8, &temp32);
	printf("CounterInternalTriggerLastValue = %lu at index %d\n", temp32, temp8);

	PDMIR_GetCounterTriggerInLastValue(sn[0], &temp8, &temp32);
	printf("CounterTriggerInLastValue = %lu at index %d\n", temp32, temp8);

	PDMIR_GetCounterValidGateLastValue(sn[0], &temp8, &temp32);
	printf("CounterValidGateLastValue = %lu at index %d\n", temp32, temp8);
	
	manageResult(PDMIR_GetAllCountersLastValue(sn[0], &temp8, &temp32, &temp32_2, &temp32_3, &temp32_4, &temp32_5));
	printf("CounterAvalanche = %lu at index %d\n", temp32, temp8);
	printf("CounterValidGate = %lu at index %d\n", temp32_2, temp8);
	printf("CounterAuxIn = %lu at index %d\n", temp32_3, temp8);
	printf("CounterTriggerIn = %lu at index %d\n", temp32_4, temp8);
	printf("CounterInternalTrigger = %lu at index %d\n", temp32_5, temp8);
		
	PDMIR_GetCounterAuxInValueAtIndex(sn[0], &temp8, &temp32);
	printf("CounterAuxInValueAtIndex = %lu at index %d\n", temp32, temp8);

	PDMIR_GetCounterInternalTriggerValueAtIndex(sn[0], &temp8, &temp32);
	printf("CounterInternalTriggerValueAtIndex = %lu at index %d\n", temp32, temp8);

	PDMIR_GetCounterAvalancheValueAtIndex(sn[0], &temp8, &temp32);
	printf("CounterAvalancheValueAtIndex = %lu at index %d\n", temp32, temp8);

	PDMIR_GetCounterTriggerInValueAtIndex(sn[0], &temp8, &temp32);
	printf("CounterTriggerInValueAtIndex = %lu at index %d\n", temp32, temp8);

	PDMIR_GetCounterValidGateValueAtIndex(sn[0], &temp8, &temp32);
	printf("CounterValidGateValueAtIndex = %lu at index %d\n", temp32, temp8);

	printf("\n*** GET CURRENT CONFIG\n");
	manageResult(PDMIR_GetCurrentConfig(sn[0], &config));
	printf("Amplitude = %d\n", config.Amplitude);
	printf("Hold Off = %lu\n", config.HoldOff);
	printf("Hold Off Level = %s\n", (config.HoldOffLevel == HOLDOFFTYPE_EDGE ? "EDGE" : "LEVEL"));
	printf("Temperature = %d\n", config.Temperature);
	printf("gateMode = %d\n", config.GateMode);
	printf("gateShape = %d\n", config.GateShape);
	printf("TTLout = %d\n", config.TtlOut);
	printf("NIMout = %d\n", config.NimOut);
	PDMIR_GetModuleStatus(sn[0], &moduleStatus, &moduleTemperature, &moduleSpad, &moduleGate, &warningUp, &errorModule);
	printf("moduleStatus = %d\n", moduleStatus);
	if (errorModule)
	{
		char temp[300];
		PDMIR_ErrorTranslator(errorModule, temp);
		printf("%s", temp);
	}

	printf("\n*** CONFIGURATIONS\n");

	UINT8 x;
	for (x = 1; x <= 10; x++) {
		RESULT res = PDMIR_GetConfigX(sn[0], x, &config, configName);
		if (res != RESULT_OK) {
			printf("config %d not saved\n", x);
		}
		else {
			printf("CONFIGURATION %d\n", x);
			printf("name = %s\n", configName);

			printf("errorModule = %d\n", res);
			res = PDMIR_GetModuleInfo(sn[0], &info);
			printf("errorGetModuleInfo = %d\n", res);
			printf("Amplitude = %d\n", config.Amplitude);
			printf("Hold Off = %lu\n", config.HoldOff);
			printf("Hold Off Level = %s\n", (config.HoldOffLevel == HOLDOFFTYPE_EDGE ? "EDGE" : "LEVEL"));
			printf("Temperature = %d\n", config.Temperature);
			printf("gateMode = %d\n", config.GateMode);
			printf("gateShape = %d\n", config.GateShape);
			printf("TTLout = %d\n", config.TtlOut);
			printf("NIMout = %d\n", config.NimOut);
			printf("DelayNimOut = %d ns\n", config.DelayNimOut);
			printf("DelayAuxIn = %d ns\n", config.DelayAuxIn);
			printf("DelayTriggerIn = %d ns\n", config.DelayTriggerIn);
			printf("DelayTriggerInternal = %d ns\n", config.DelayTriggerInternal);
			printf("DelayTtlOut = %d ns\n", config.DelayTtlOut);
			printf("DelayGate = %d ns\n", config.DelayGate);
			printf("CounterIntegrationTime = %d ms\n", config.CounterIntegrationTime);
			printf("CounterAvalancheStatus = %d\n", config.CounterAvalancheStatus);
			printf("CounterAuxInStatus = %d\n", config.CounterAuxInStatus);
			printf("CounterInternalTriggerStatus = %d\n", config.CounterInternalTriggerStatus);
			printf("CounterTriggerInStatus = %d\n", config.CounterTriggerInStatus);
			printf("CounterValidGateStatus = %d\n", config.CounterValidGateStatus);
		}
		printf("\n");
	}

	printf("get powerUp config \n");
	manageResult(PDMIR_GetConfigPowerUp(sn[0], &x));
	printf("config = %d\n", x);

	PDMIR_CloseCommunication(sn[0]);

	return exit();
}

